@@ -1,4 +1,6 @@
-import os, sys, select, socket, json, threading, random, discord
+import select, socket, json, threading, random, discord, re
+from os import path, devnull, system, remove
+from sys import stdout, __stdout__
 from time import sleep, time
 from typing import Callable
 
@@ -43,27 +45,28 @@ class Attachment:
         return requests.get(self.url)
 
     def save(self, path: str) -> str:
-        if not os.path.exists(path): return ""
+        if not path.exists(path): return ""
         tmp = self.filename
         n = 1
-        while os.path.exists(os.path.join(path, tmp)):
+        while path.exists(path.join(path, tmp)):
             tmp = "".join((".".join(self.filename.split(".")[:-1]), f"({n}).", self.filename.split(".")[-1]))
             n += 1
         self.filename = tmp
         file = self.download()
-        with open(os.path.join(path, self.filename), "wb") as f:
+        with open(path.join(path, self.filename), "wb") as f:
             f.write(file.content)
-        return os.path.join(path, self.filename)
+        return path.join(path, self.filename)
     
     def to_json(self) -> dict:
         return {"filename":self.filename, "size":self.size, "url":self.url}
 
 class Message:
     """Message object used by the api"""
+    USER_REGX = r"(<@![0-9]+>){1}"
     def from_json(json):
         return Message(json["sender"], json["content"], json["channel"], [Attachment.from_json(attachment) for attachment in json["attachments"]] if json["attachments"] is not None else [], json["called"])
 
-    def __init__(self, sender: str, content: str, channel: str, attachments: str, called: str) -> None:
+    def __init__(self, sender: str, content: str, channel: str, attachments: list, called: str) -> None:
         self.sender = sender
         self.content = content
         self.channel = channel
@@ -72,19 +75,33 @@ class Message:
 
     def add_called(self, called: str) -> None:
         self.called = called
-        
+
+    def contains_user(self) -> bool:
+        return len(re.findall(Message.USER_REGX, self.content)) > 0
+
+    def get_contained_user_id(self) -> str:
+        if not self.contains_user():
+            return ""
+        if " " in self.content:
+            content = self.content.split(' ')
+            for item in content:
+                if "<@" in item:
+                    return item.replace("<@!", "").replace(">", "")
+    
     def has_attachments(self) -> bool:
-        return len(self.attachments) > 0
+        return len(self.attachments) == 1
 
     def to_json(self) -> dict:
         return {"sender":self.sender, "content":self.content, "channel":self.channel, "called":self.called, "attachments":[attachment.to_json() for attachment in self.attachments] if len(self.attachments) > 0 else None}
 
 def blockPrint() -> None:
-    sys.stdout = open(os.devnull, 'w')
+    global stdout
+    stdout = open(devnull, 'w')
 
 def enablePrint() -> None:
-    sys.stdout.close()
-    sys.stdout = sys.__stdout__
+    global stdout
+    stdout.close()
+    stdout = __stdout__
 
 NOTHING = 0
 USER_INPUT = 1
@@ -147,16 +164,21 @@ class API:
             print(ex)
             return None
     
+    def _get_copy_function_list(self):
+        ret = []
+        for item in self.created_function_list:
+            ret.append(item)
+        return ret
+
     def _re_init_commands(self) -> None:
-        from copy import deepcopy
-        tmp = deepcopy(self.created_function_list)
+        tmp = self._get_copy_function_list()
         for item in tmp:
             self.create_function(*item)
         del tmp
 
     def validate(self, timeout: int = -1) -> bool:
         """Validates with the bot, and starts the listener loop, if validation is finished. Returns false, if validation timed out
-        Time out can be set, so it won't halt the program for ever, if no bot is present. (The timeout will only work for the first validation.)
+        Timeout can be set, so it won't halt the program for ever, if no bot is present. (The timeout will only work for the first validation.)
         If the timeout is set to -1, the validation will be in a new thread, and always return true.
         """
         if timeout is not None and timeout == -1:
@@ -174,7 +196,7 @@ class API:
             try:
                 self.socket.connect((self.ip, self.port))
                 break
-            except ConnectionRefusedError: pass
+            except: pass
             if (timeout is not None and timeout > 0 and time() - start > timeout) or not self.running:
                 return False
         self._send({"Command":self.name, "Value": self.key})
@@ -213,7 +235,8 @@ class API:
     def update(self, _) -> None:
         """Trys to update the API with PIP, and calls the given update function if there is one avaleable.
         """
-        os.system("pip install --user --upgrade smdb_api")
+        system("pip install --user --upgrade smdb_api > pip.txt")
+        remove("pip.txt")
         if self.update_function is not None:
             self.update_function()
 
@@ -244,7 +267,7 @@ class API:
     def send_message(self, message: str, destination: str = None, file_path: str = None) -> bool:
         """Sends a message trough the discord bot.
         """
-        msg = Message("API", message, destination, [Attachment(file_path.split("/")[-1], file_path, os.path.getsize(file_path))] if file_path is not None else [], "API")
+        msg = Message("API", message, destination, [Attachment(file_path.split("/")[-1], file_path, path.getsize(file_path))] if file_path is not None else [], "API")
         if self.valid:
             self.sending = True
             self._send({"Command":"Send", 'Value': msg.to_json()})
@@ -320,11 +343,11 @@ class API:
         """Creates a function in the connected bot when validated.
         """
         while self.sending:
-            sleep(random.randrange(1, 3))
+            sleep(random.randrange(0, 5))
+        self.sending = True
         if [name, help_text, callback] not in self.created_function_list:
             self.created_function_list.append([name, help_text, callback])
         if not self.valid: return
-        self.sending = True
         self._send({"Command":"Create", "Value": [name, help_text, name]})
         while self.buffer == []:
             sleep(0.1)
